package com.akamai.media.demo4;

import com.akamai.media.IPlayerEventsListener;
import com.akamai.media.VideoPlayerContainer;
import com.akamai.media.VideoPlayerView;
import com.akamai.media.VideoPlayerViewSoftware;

import android.app.Activity;
import android.content.DialogInterface;
import android.content.SharedPreferences;
import android.media.AudioManager;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.preference.PreferenceManager;
import android.util.Log;
import android.view.MotionEvent;
import android.view.View;
import android.view.Window;
import android.view.View.OnTouchListener;
import android.view.animation.AnimationUtils;
import android.widget.ImageView;
import android.widget.ProgressBar;
import android.widget.TextView;
import android.widget.Toast;

public class PlayerVideoActivity  extends Activity implements OnTouchListener, DialogInterface.OnCancelListener, IPlayerEventsListener
{
    private VideoPlayerView mVideoView;
    private VideoPlayerContainer mVideoContainer;
    
    private ImageView mPlayerDefaultImg;
    private PlayerPlayControls mPlayerControls;
    private PlayerSeekbarControls mPlayerSeekbar;
    private ProgressBar mProgressBar;
    private Boolean mShowingControlBar = false;
    private TextView mPlayerText;
    private String mUrl = "";
    private Boolean mPlaying = false;
    private Boolean mIsAudioOnlyLastValue = false;
    private int mCurrentPosition = 0;
 	private boolean mWasPlaying = false;
    
    private final int COMMAND_FINISH = 0;
    private final int COMMAND_PLAYING = 1;
    private final int COMMAND_ERROR = 2;
    private final int COMMAND_START_BUFFERING = 3;
    private final int COMMAND_END_BUFFERING = 4;
    private final int COMMAND_UPDATE_POSITION = 5;
    private final int COMMAND_HIDE_CONTROLBAR = 6;
    
    private final int SHOWTIME_CONTROLBAR = 10000;
    
    //private final String MEDIA_ANALYTICS_CONFIG_URL = "http://ma1-r.analytics.edgesuite.net/config/beacon-1712.xml";
    
    public Handler mCommandsHandler = new Handler()
	{
		@Override 
		public void handleMessage(Message msg)
		{
			
			switch(msg.what)
			{
			
			case COMMAND_START_BUFFERING:
				Log.d("EVENTS", "Message: START BUFFERING");
				mProgressBar.setVisibility(View.VISIBLE);
				break;
				
			case COMMAND_END_BUFFERING:
				Log.d("EVENTS", "Message: END BUFFERING");
				mProgressBar.setVisibility(View.GONE);
				break;
				
			case COMMAND_FINISH:
				Log.d("EVENTS", "Message: FINISH BUFFERING");
				break;
			
			case COMMAND_PLAYING:
				Log.d("EVENTS", "Message: PLAYING BUFFERING");
				if(!mPlaying)
				{
					mPlaying = true;  
					
					mProgressBar.setVisibility(View.GONE);
					
					int duration = mVideoView.getDuration();
					mPlayerSeekbar.setMax(duration);
					if(mUrl.startsWith("rtmp://"))
						mPlayerSeekbar.DisableSeekBar();
					
					mIsAudioOnlyLastValue = mVideoView.isAudioOnly();
					UpdatePlayerControls();
				}
				break;
				
			case COMMAND_ERROR:
				Log.d("EVENTS", "Message: ERROR");
				mProgressBar.setVisibility(View.GONE);
				
				Toast.makeText(PlayerVideoActivity.this, "Error event raised. Last HTTP code: " + mVideoView.getLastHttpErrorCode(), Toast.LENGTH_LONG).show();
				//finish();
				break;
				
			case COMMAND_HIDE_CONTROLBAR:
				if(mPlayerControls.isTouching() || mPlayerSeekbar.isTouching())
					mCommandsHandler.sendEmptyMessageDelayed(COMMAND_HIDE_CONTROLBAR, SHOWTIME_CONTROLBAR);
				else
					showControlsBar(false);
				break;
				
			case COMMAND_UPDATE_POSITION:
				Log.d("EVENTS", "Message: UPDATE POSITION");
				{
					int iPlayerPosition = mVideoView.getTimePosition();
					Log.d("EVENTS", "Position: " + iPlayerPosition);
					
					if(iPlayerPosition >= 0)
					{
						mPlayerSeekbar.setProgress(iPlayerPosition);
					}
				}
				
				// We can switch to an audio only stream automatically because network/cpu conditions
				if(mIsAudioOnlyLastValue != mVideoView.isAudioOnly())
				{
					mIsAudioOnlyLastValue = mVideoView.isAudioOnly();
					UpdatePlayerControls();			
				}
				
				if(mVideoView.isAudioOnly())
				{
					mCommandsHandler.sendEmptyMessageDelayed(COMMAND_UPDATE_POSITION, 1000);
				}
				mPlayerText.setText(String.format("%d bps", mVideoView.getCurrentBitrate()));
				break;
				
			default:
				break;
			}

			super.handleMessage(msg);
		}
	};
	
	private void UpdatePlayerControls()
	{
		if(mIsAudioOnlyLastValue)
		{
			((View)mVideoView).setVisibility(View.INVISIBLE);
			mPlayerDefaultImg.setVisibility(View.VISIBLE);
			showControlsBar(true);
			
			mCommandsHandler.sendEmptyMessageDelayed(COMMAND_UPDATE_POSITION, 1000);
		}
		else
		{
			((View)mVideoView).setVisibility(View.VISIBLE);
			mPlayerDefaultImg.setVisibility(View.INVISIBLE);
		}
	}
	
	public boolean onPlayerEvent(int iInEventType)
	{
		String sEventType = "Unknown";
		switch(iInEventType)
		{
			case IPlayerEventsListener.PLAYER_EVENT_TYPE_LOADING: //8
				sEventType = "PLAYER_EVENT_TYPE_LOADING";
				break;
				
			case IPlayerEventsListener.PLAYER_EVENT_TYPE_FINISHED: //1
				sEventType = "PLAYER_EVENT_TYPE_FINISHED";
				mCommandsHandler.sendEmptyMessage(COMMAND_FINISH);
				break;
				
			case IPlayerEventsListener.PLAYER_EVENT_TYPE_START_PLAYING: //2
				sEventType = "PLAYER_EVENT_TYPE_START_PLAYING";
				mCommandsHandler.sendEmptyMessage(COMMAND_PLAYING);
				break;
				
			case IPlayerEventsListener.PLAYER_EVENT_TYPE_ERROR: //3
				sEventType = "PLAYER_EVENT_TYPE_ERROR";
				mCommandsHandler.sendEmptyMessage(COMMAND_ERROR);
				break;
				
			case IPlayerEventsListener.PLAYER_EVENT_TYPE_POSITION_UPDATE: //0
				sEventType = "PLAYER_EVENT_TYPE_POSITION_UPDATE";
				mCommandsHandler.sendEmptyMessage(COMMAND_UPDATE_POSITION);
				break;
				
			case IPlayerEventsListener.PLAYER_EVENT_TYPE_START_REBUFFERING: //4
				sEventType = "PLAYER_EVENT_TYPE_START_REBUFFERING";
				mCommandsHandler.sendEmptyMessage(COMMAND_START_BUFFERING);
				break;
				
			case IPlayerEventsListener.PLAYER_EVENT_TYPE_END_REBUFFERING: //5
				sEventType = "PLAYER_EVENT_TYPE_END_REBUFFERING";
				mCommandsHandler.sendEmptyMessage(COMMAND_END_BUFFERING);
				break;
				
			case IPlayerEventsListener.PLAYER_EVENT_TYPE_SWITCH_REQUESTED: //6
				sEventType = "PLAYER_EVENT_TYPE_SWITCH_REQUESTED";
				break;
				
			case IPlayerEventsListener.PLAYER_EVENT_TYPE_SWITCH: //7
				sEventType = "PLAYER_EVENT_TYPE_SWITCH";
				break;
				
			case IPlayerEventsListener.PLAYER_EVENT_TYPE_FULLSCREEN_MODE_CHANGE: //11
				sEventType = "PLAYER_EVENT_TYPE_FULLSCREEN_MODE_CHANGE";
				break;
				
			case IPlayerEventsListener.PLAYER_EVENT_TYPE_SIZE_CHANGE: //12
				sEventType = "PLAYER_EVENT_TYPE_SIZE_CHANGE";
				break;
				
			default:
				break;
		}
		Log.d("Player", "Event Type: " + sEventType + " (" + iInEventType + "), isSeeking: " + mVideoView.isSeeking());
		
		
		return true;
	}

	@Override
	public boolean onPlayerExtendedEvent(int iInEventType, int arg1, int arg2) 
	{
		if(iInEventType == IPlayerEventsListener.PLAYER_EXTENDED_EVENT_SEEKING_SUCCEDEED)
			Log.d("Player", "Event Type: PLAYER_EXTENDED_EVENT_SEEKING_SUCCEDEED (" + iInEventType + ")");
		
		return true;
	}
    
    public void playPause()
    {    	
    	if(mVideoView.isPaused())
    	{
    		if(mVideoContainer.getMode() == VideoPlayerContainer.MODE_SOFTWARE)
    		{
    			((VideoPlayerViewSoftware) mVideoView).setVisibility(View.VISIBLE);
    		}
    		mVideoView.resume();
    	}
    	else
    	{
    		if(mVideoContainer.getMode() == VideoPlayerContainer.MODE_SOFTWARE)
    		{
    			((VideoPlayerViewSoftware) mVideoView).setVisibility(View.INVISIBLE);
    		}
    		mVideoView.pause();
    	}
    
    	mPlayerControls.setStatus(!mVideoView.isPaused());
    }
    
    public void rewind()
    {
    	try
    	{
    		mVideoView.switchBitrateDown();
    	}
    	catch(Exception e)
    	{
    		Log.d("PlayerVideoActivity", "There was an issue while switching to the lower bitrate. Reason: " + e.getMessage());
    	}
    }
    
    public void forward()
    {
    	try
    	{
    		mVideoView.switchBitrateUp();
    	}
    	catch(Exception e)
    	{
    		Log.d("PlayerVideoActivity", "There was an issue while switching to the higher bitrate. Reason: " + e.getMessage());
    	}
    }
    
    public void seek(int position)
    {
    	if(mVideoView.isLive())
    		position = position * (int)mVideoView.getDVRLength()/100;
    		
    	mVideoView.seek(position, 0);
    }
    
    public void toggleFullscreen()
    {
    	int mode = mVideoView.getFullScreenMode();
    	Log.d("Player", "Fullscreen Mode: " + mode);
    	
    	if(mode == VideoPlayerView.FULLSCREEN_MODE_NONE)
    		mVideoView.setFullScreenMode(VideoPlayerView.FULLSCREEN_MODE_KEEP_ASPECT_RATIO);
    	else if(mode == VideoPlayerView.FULLSCREEN_MODE_KEEP_ASPECT_RATIO)
    		mVideoView.setFullScreenMode(VideoPlayerView.FULLSCREEN_MODE_TOTAL);
    	else if(mode == VideoPlayerView.FULLSCREEN_MODE_TOTAL)
    		mVideoView.setFullScreenMode(VideoPlayerView.FULLSCREEN_MODE_NONE);
    }

    

    public void showControlsBar(Boolean show)
    {
    	mShowingControlBar = show;
    	
    	if(mShowingControlBar)
    	{
    		mPlayerControls.setStatus(!mVideoView.isPaused());
    		mPlayerControls.startAnimation(AnimationUtils.loadAnimation(this, R.anim.playercontrolsfadeout));
    		mPlayerControls.setVisibility(View.VISIBLE);
    		mPlayerSeekbar.startAnimation(AnimationUtils.loadAnimation(this, R.anim.playercontrolsfadeout));
    		mPlayerSeekbar.setVisibility(View.VISIBLE);
    	}
    	else
    	{
    		mPlayerControls.startAnimation(AnimationUtils.loadAnimation(this, R.anim.playercontrolsfadein));
    		mPlayerControls.setVisibility(View.INVISIBLE);
    		mPlayerSeekbar.startAnimation(AnimationUtils.loadAnimation(this, R.anim.playercontrolsfadein));
    		mPlayerSeekbar.setVisibility(View.INVISIBLE);
    	}
    }
    
    @Override
    protected void onCreate(Bundle savedInstanceState) 
    {
        super.onCreate(savedInstanceState);
     
        // Hide the title and notification bars
        requestWindowFeature(Window.FEATURE_NO_TITLE);  
        setContentView(R.layout.playerviewlayout);
        
        // Get Player Controls
        mPlayerControls = (PlayerPlayControls) findViewById(R.id.playerControls);
        mPlayerSeekbar = (PlayerSeekbarControls) findViewById(R.id.playerSeekBar);
        mPlayerText = (TextView) findViewById(R.id.playerTextInfo); 
        
        // Configure the GL surface
        mVideoContainer = (VideoPlayerContainer) findViewById(R.id.playerViewCtrl);
        
        // Get default image control
        mPlayerDefaultImg = (ImageView) findViewById(R.id.playerDefaultImg);
        
        // Progress bar
        mProgressBar = (ProgressBar) findViewById(R.id.progressBar);
      
        // Configure the render object
        Bundle oBundle = getIntent().getExtras();
        mUrl = oBundle.getString("url");
        
        // Configure the video player
        configureVideoPlayer();
        
        // Configure the audio
        setVolumeControlStream(AudioManager.STREAM_MUSIC);
       
        // Play the stream
        mVideoView.playUrl(mUrl);
    }

    private void configureVideoPlayer() 
	{
		SharedPreferences settings = PreferenceManager.getDefaultSharedPreferences(this);
		String sPlayerMode = settings.getString(PlayerConfiguration.PREFS_PLAYER_WORKING_MODE, "10");
        Integer iPlayerMode = Integer.valueOf(sPlayerMode);
        
        String sMaxBitrateValue = settings.getString(PlayerConfiguration.PREFS_MAX_BITRATE, PlayerConfiguration.PREFS_MAX_BITRATE_DEFAULT_VALUE);
        Integer iMaxBitrateValue = Integer.valueOf(sMaxBitrateValue);
        
        String sStartingAlgorithm = settings.getString(PlayerConfiguration.PREFS_HLS_START_ALGORITHM, PlayerConfiguration.PREFS_HLS_START_ALGORITHM_DEFAULT_VALUE);
        Integer iStartingAlgorithm = Integer.valueOf(sStartingAlgorithm);
        
        String sNetsessionMode = settings.getString(PlayerConfiguration.PREFS_NETSESSION_MODE, PlayerConfiguration.PREFS_NETSESSION_MODE_DEFAULT_VALUE);
        Integer iNetsessionMode = Integer.valueOf(sNetsessionMode);
        
        String sVideoQuality = settings.getString(PlayerConfiguration.PREFS_VIDEO_QUALITY, PlayerConfiguration.PREFS_VIDEO_QUALITY_DEFAULT_VALUE);
        Integer iVideoQuality = Integer.valueOf(sVideoQuality);
        
        String sDropPackets = settings.getString(PlayerConfiguration.PREFS_DROP_WRONG_TIMESTAMPS_MODE, PlayerConfiguration.PREFS_DROP_WRONG_TIMESTAMPS_MODE_DEFAULT_VALUE);
        Integer iDropPackets = Integer.valueOf(sDropPackets);
        
        String sBufferSize = settings.getString(PlayerConfiguration.PREFS_BUFFER_SIZE, PlayerConfiguration.PREFS_BUFFER_SIZE_DEFAULT_VALUE);
        Integer iBufferSize = Integer.valueOf(sBufferSize);
        
        boolean automaticBitrate = settings.getBoolean(PlayerConfiguration.PREFS_AUTO_SWITCHING, true);
        boolean dropFrames = settings.getBoolean(PlayerConfiguration.PREFS_DROP_FRAMES, true);
        boolean adjustTimestamps = settings.getBoolean(PlayerConfiguration.PREFS_ADJUST_TIMESTAMPS, true);
        boolean avoidAudioStream = settings.getBoolean(PlayerConfiguration.PREFS_AVOID_AUDIO_STREAM, true);
        boolean autoBufferMode = settings.getBoolean(PlayerConfiguration.PREFS_AUTO_BUFFER_MODE, true);
        boolean useStartBuffer = settings.getBoolean(PlayerConfiguration.PREFS_USE_START_BUFFER, false);
        boolean disableDynamicAudio = settings.getBoolean(PlayerConfiguration.PREFS_DISABLE_DYNAMIC_AUDIO, false);
        boolean useMultithread = settings.getBoolean(PlayerConfiguration.PREFS_USE_MULTITHREAD, false);
        boolean forceFormatChange = settings.getBoolean(PlayerConfiguration.PREFS_FORCE_FORMAT_CHANGE, false);
        
        
        // Set the player mode
        mVideoContainer.setMode(iPlayerMode);
        mVideoView = mVideoContainer.getVideoPlayer();
        
        mVideoView.setEventsListener(this);
        ((View)mVideoView).setKeepScreenOn(true);
        ((View)mVideoView).setOnTouchListener(this);
        
        mVideoView.setLicense("HHMALdN_yu_KrtsBWI7qH3Nn0N_KidMA6MtJzll5xlCyWcv37gy_UEOLv4mPxN1vXUIPWjxiRX1ZVUauvrg-5Q==");
        Log.d("PlayerView", "License package: " + mVideoView.getLicensePackageName());
        Log.d("PlayerView", "License expiration date: " + mVideoView.getLicenseExpirationDate());
        
        
        mVideoView.setUseMultiThread(useMultithread);
        mVideoView.setDebugingActive(false);
        mVideoView.setLogEnabled(true);
        
        mVideoView.setHLSStartingAlgorithm(iStartingAlgorithm);
        mVideoView.setNetSessionMode(iNetsessionMode);
        mVideoView.setVideoQuality(iVideoQuality);
        mVideoView.setManualSwitching(!automaticBitrate);
        
        mVideoView.setDropFrames(dropFrames);
        mVideoView.setDropWrongTimestampPacketsMode(iDropPackets);
        mVideoView.setAdjustTimestamps(adjustTimestamps);
        mVideoView.setMaxBitrate(iMaxBitrateValue);
        mVideoView.setAvoidAudioOnlyStreams(avoidAudioStream);
        mVideoView.setForceFormatChange(forceFormatChange);
        
        if(autoBufferMode)
        	mVideoView.setRebufferingMode(VideoPlayerView.MEDIA_REBUFFERING_MODE_AUTOMATIC);
        else
        {
        	mVideoView.setRebufferingMode(VideoPlayerView.MEDIA_REBUFFERING_MODE_MANUAL);
        	mVideoView.setRebufferingSize(iBufferSize*1024);
        }
        mVideoView.setUseBufferingWhenStarting(!useStartBuffer);
        mVideoView.setDisableDynamicAudioFeature(!disableDynamicAudio);
	}
    
    @Override
    protected void onPause() 
    {
    	if (mVideoView.isPlaying() || mVideoView.isPaused())
        {
        	mCurrentPosition = mVideoView.getTimePosition();
        	mWasPlaying = true;
        }
    	
        mVideoView.stop();
        super.onPause();
    }

    @Override
    protected void onResume() 
    {
    	if (mWasPlaying)
    	{
    		mVideoView.playUrl(mUrl, mCurrentPosition);
    		mWasPlaying = false;
    	}
    	
        super.onResume();
    }
        
    
    @Override
    protected void onDestroy() 
    {	
        mVideoView.stop();
        mVideoView.onDestroy();
        super.onDestroy();
    }
    
	@Override
	public boolean onTouch(View v, MotionEvent event) 
	{
		if(event.getAction() == MotionEvent.ACTION_DOWN && v == mVideoView)
		{
			showControlsBar(!mShowingControlBar);
		}	
		
		return false;
	}

	@Override
	public void onCancel(DialogInterface dialog) {
		finish();
	}

}
